@@ -1,6 +1,8 @@
 import arrow
 import csv
 import datetime
+import ics.attendee
+import ics.event
 import nytid.schedules
 import re
 import requests
@@ -9,6 +11,47 @@ SIGNUP_SHEET_HEADER = [
   "Event", "Start", "End", "#Rooms",
   "#Needed TAs"
 ]
+class EventFromCSV(ics.event.Event):
+  """A class to create an ics.event.Event from an event in CSV format"""
+  def __init__(self, csv_row):
+    """
+    Input: a row from a calendar in CSV format (e.g. the sign-up sheet).
+    """
+    attribute_map = {
+      SIGNUP_SHEET_HEADER.index("Event"): "name",
+      SIGNUP_SHEET_HEADER.index("Start"): "begin",
+      SIGNUP_SHEET_HEADER.index("End"): "end",
+      SIGNUP_SHEET_HEADER.index("#Rooms"): "description",
+      SIGNUP_SHEET_HEADER.index("#Needed TAs"): "description"
+    }
+    for idx in range(max(attribute_map.keys()), len(csv_row)):
+      attribute_map[idx] = "attendees"
+
+    kwargs = dict()
+
+    for column, attribute in attribute_map.items():
+      try:
+        value = csv_row[column]
+
+        if attribute == "description":
+          if attribute not in kwargs:
+            kwargs[attribute] = value
+          else:
+            kwargs[attribute] += "\n" + value
+        elif attribute == "attendees":
+          value = ics.attendee.Attendee(f"{value}@kth.se")
+          if attribute not in kwargs:
+            value = [value]
+          else:
+            value = kwargs[attribute] + [value]
+        elif attribute in ["begin", "end"]:
+          value = arrow.get(value, tzinfo="local")
+
+        kwargs[attribute] = value
+      except AttributeError:
+        pass
+
+    super().__init__(**kwargs)
 def needed_TAs(event):
   """
   Takes an event and returns the number of TAs needed
@@ -73,7 +116,7 @@ def read_signup_sheet_from_file(filename):
   """
   with open(filename, "r") as f:
     csvfile = csv.reader(f)
-    return list(csvfile)[1:]
+    return list(filter(any, list(csvfile)[1:]))
 def read_signup_sheet_from_url(url):
   """
   Input: url is a string containing the URL of the CSV file of the sign-up 
@@ -87,7 +130,7 @@ def read_signup_sheet_from_url(url):
 
   response.encoding = response.apparent_encoding
   csvdata = response.text.splitlines()
-  return list(csv.reader(csvdata))[1:]
+  return list(filter(any, list(csv.reader(csvdata))[1:]))
 def google_sheet_to_csv_url(share_url):
   """
   Input: The share URL of a Google Sheets sheet.
@@ -107,6 +150,31 @@ def get_TAs_from_csv(csv_row):
   Output: returns the list of signed TAs.
   """
   return csv_row[len(SIGNUP_SHEET_HEADER):]
+def hours_per_student(csv_rows):
+  """
+  Input: The schedule as rows of CSV data as from csv.reader.
+
+  Output: A dictionary mapping event type to the total number of hours per 
+  student.
+  """
+
+  start_index = SIGNUP_SHEET_HEADER.index("Start")
+  end_index = SIGNUP_SHEET_HEADER.index("End")
+  event_index = SIGNUP_SHEET_HEADER.index("Event")
+
+  event_hours = dict()
+
+  for row in csv_rows:
+    time = round_time(arrow.get(row[end_index], "YYYY-MM-DD HH:mm") - \
+      arrow.get(row[start_index], "YYYY-MM-DD HH:mm"))
+    event_type = row[event_index]
+
+    if event_type not in event_hours:
+      event_hours[event_type] = time
+    else:
+      event_hours[event_type] += time
+
+  return event_hours
 def hours_per_TA(csv_rows):
   """
   Input: Rows of CSV data as from csv.reader.
@@ -245,14 +313,32 @@ def compute_amanuensis_data(csv_rows, low_percentage=0.1, min_days=25):
 
     hours = ta_hours[ta].total_seconds()/60/60
 
-    earliest_month = arrow.get(
-      f"{earliest_date.year}-{earliest_date.month:02d}-01",
-      "YYYY-MM-DD")
-    latest_month = arrow.get(
-      f"{latest_date.year}-{latest_date.month:02d}-01",
-      "YYYY-MM-DD").shift(months=1, seconds=-1)
+    july = arrow.get(earliest_date.year, 7, 1)
 
-    if compute_percentage(earliest_month, latest_month, hours) >= low_percentage:
+    if earliest_date < july:
+      semester_start = arrow.get(earliest_date.year, 1, 1)
+      semester_end = arrow.get(earliest_date.year, 6, 30)
+    else:
+      semester_start = arrow.get(earliest_date.year, 8, 1)
+      semester_end = arrow.get(earliest_date.year+1, 1, 31)
+
+    earliest_month = arrow.get(earliest_date.year, earliest_date.month, 1)
+    latest_month = arrow.get(
+      latest_date.year, latest_date.month, 1).shift(months=1, seconds=-1)
+
+    if latest_date > semester_end:
+      semester_end = latest_month
+
+    if compute_percentage(semester_start, semester_end, hours) >= low_percentage:
+      earliest_date = semester_start
+      latest_date = semester_end
+    elif compute_percentage(semester_start, latest_month, hours) >= low_percentage:
+      earliest_date = earliest_month
+      latest_date = latest_month
+    elif compute_percentage(earliest_month, semester_end, hours) >= low_percentage:
+      earliest_date = earliest_month
+      latest_date = latest_month
+    elif compute_percentage(earliest_month, latest_month, hours) >= low_percentage:
       earliest_date = earliest_month
       latest_date = latest_month
     elif compute_percentage(earliest_date, latest_month, hours) >= low_percentage:
@@ -274,8 +360,7 @@ def compute_percentage(start, end, hours):
   Output: a float in the interval [0, 1], which is the percentage of full time.
   """
   days = (end - start).total_seconds()/60/60/24
-  full_time = 40 * days / 7
-  return hours / full_time
+  return (hours / 1600) * (365 / days)
 
 def main():
   COURSES = {
